@@ -25,6 +25,66 @@
 #include "mm-errors-types.h"
 #include "mm-common-helpers.h"
 
+static gint
+_enum_from_string (GType type,
+                   const gchar *str,
+                   gint error_value,
+                   GError **error)
+{
+    GEnumClass *enum_class;
+    gint value;
+    guint i;
+
+    enum_class = G_ENUM_CLASS (g_type_class_ref (type));
+
+    for (i = 0; enum_class->values[i].value_nick; i++) {
+        if (!g_ascii_strcasecmp (str, enum_class->values[i].value_nick)) {
+            value = enum_class->values[i].value;
+            g_type_class_unref (enum_class);
+            return value;
+        }
+    }
+
+    g_set_error (error,
+                 MM_CORE_ERROR,
+                 MM_CORE_ERROR_INVALID_ARGS,
+                 "Couldn't match '%s' with a valid %s value",
+                 str,
+                 g_type_name (type));
+    g_type_class_unref (enum_class);
+    return error_value;
+}
+
+static guint
+_flags_from_string (GType type,
+                    const gchar *str,
+                    guint error_value,
+                    GError **error)
+{
+    GFlagsClass *flags_class;
+    guint value;
+    guint i;
+
+    flags_class = G_FLAGS_CLASS (g_type_class_ref (type));
+
+    for (i = 0; flags_class->values[i].value_nick; i++) {
+        if (!g_ascii_strcasecmp (str, flags_class->values[i].value_nick)) {
+            value = flags_class->values[i].value;
+            g_type_class_unref (flags_class);
+            return value;
+        }
+    }
+
+    g_set_error (error,
+                 MM_CORE_ERROR,
+                 MM_CORE_ERROR_INVALID_ARGS,
+                 "Couldn't match '%s' with a valid %s value",
+                 str,
+                 g_type_name (type));
+    g_type_class_unref (flags_class);
+    return error_value;
+}
+
 gchar *
 mm_common_build_capabilities_string (const MMModemCapability *capabilities,
                                      guint n_capabilities)
@@ -662,12 +722,52 @@ mm_common_bands_garray_cmp (GArray *a, GArray *b)
     return !different;
 }
 
+gboolean
+mm_common_bands_garray_lookup (GArray      *array,
+                               MMModemBand  value)
+{
+    guint i;
+
+    for (i = 0; i < array->len; i++) {
+        if (value == g_array_index (array, MMModemBand, i))
+            return TRUE;
+    }
+
+    return FALSE;
+}
+
 void
 mm_common_bands_garray_sort (GArray *array)
 {
     g_array_sort (array, (GCompareFunc) cmp_band);
 }
 
+gboolean
+mm_common_band_is_gsm (MMModemBand band)
+{
+    return ((band >= MM_MODEM_BAND_EGSM && band <= MM_MODEM_BAND_G850) ||
+            (band >= MM_MODEM_BAND_G450 && band <= MM_MODEM_BAND_G810));
+}
+
+gboolean
+mm_common_band_is_utran (MMModemBand band)
+{
+    return ((band >= MM_MODEM_BAND_UTRAN_1 && band <= MM_MODEM_BAND_UTRAN_7) ||
+            (band >= MM_MODEM_BAND_UTRAN_10 && band <= MM_MODEM_BAND_UTRAN_32));
+}
+
+gboolean
+mm_common_band_is_eutran (MMModemBand band)
+{
+    return (band >= MM_MODEM_BAND_EUTRAN_1 && band <= MM_MODEM_BAND_EUTRAN_71);
+}
+
+gboolean
+mm_common_band_is_cdma (MMModemBand band)
+{
+    return (band >= MM_MODEM_BAND_CDMA_BC0 && band <= MM_MODEM_BAND_CDMA_BC19);
+}
+
 GArray *
 mm_common_mode_combinations_variant_to_garray (GVariant *variant)
 {
@@ -762,22 +862,10 @@ MMModem3gppEpsUeModeOperation
 mm_common_get_eps_ue_mode_operation_from_string (const gchar  *str,
                                                  GError      **error)
 {
-    GEnumClass *enum_class;
-    guint       i;
-
-    enum_class = G_ENUM_CLASS (g_type_class_ref (MM_TYPE_MODEM_3GPP_EPS_UE_MODE_OPERATION));
-
-    for (i = 0; enum_class->values[i].value_nick; i++) {
-        if (!g_ascii_strcasecmp (str, enum_class->values[i].value_nick))
-            return enum_class->values[i].value;
-    }
-
-    g_set_error (error,
-                 MM_CORE_ERROR,
-                 MM_CORE_ERROR_INVALID_ARGS,
-                 "Couldn't match '%s' with a valid MMModem3gppEpsUeModeOperation value",
-                 str);
-    return MM_MODEM_3GPP_EPS_UE_MODE_OPERATION_UNKNOWN;
+    return _enum_from_string (MM_TYPE_MODEM_3GPP_EPS_UE_MODE_OPERATION,
+                              str,
+                              MM_MODEM_3GPP_EPS_UE_MODE_OPERATION_UNKNOWN,
+                              error);
 }
 
 GArray *
@@ -881,44 +969,20 @@ MMModemCdmaRmProtocol
 mm_common_get_rm_protocol_from_string (const gchar *str,
                                        GError **error)
 {
-    GEnumClass *enum_class;
-    guint i;
-
-    enum_class = G_ENUM_CLASS (g_type_class_ref (MM_TYPE_MODEM_CDMA_RM_PROTOCOL));
-
-    for (i = 0; enum_class->values[i].value_nick; i++) {
-        if (!g_ascii_strcasecmp (str, enum_class->values[i].value_nick))
-            return enum_class->values[i].value;
-    }
-
-    g_set_error (error,
-                 MM_CORE_ERROR,
-                 MM_CORE_ERROR_INVALID_ARGS,
-                 "Couldn't match '%s' with a valid MMModemCdmaRmProtocol value",
-                 str);
-    return MM_MODEM_CDMA_RM_PROTOCOL_UNKNOWN;
+    return _enum_from_string (MM_TYPE_MODEM_CDMA_RM_PROTOCOL,
+                              str,
+                              MM_MODEM_CDMA_RM_PROTOCOL_UNKNOWN,
+                              error);
 }
 
 MMBearerIpFamily
 mm_common_get_ip_type_from_string (const gchar *str,
                                    GError **error)
 {
-    GFlagsClass *flags_class;
-    guint i;
-
-    flags_class = G_FLAGS_CLASS (g_type_class_ref (MM_TYPE_BEARER_IP_FAMILY));
-
-    for (i = 0; flags_class->values[i].value_nick; i++) {
-        if (!g_ascii_strcasecmp (str, flags_class->values[i].value_nick))
-            return flags_class->values[i].value;
-    }
-
-    g_set_error (error,
-                 MM_CORE_ERROR,
-                 MM_CORE_ERROR_INVALID_ARGS,
-                 "Couldn't match '%s' with a valid MMBearerIpFamily value",
-                 str);
-    return MM_BEARER_IP_FAMILY_NONE;
+    return _flags_from_string (MM_TYPE_BEARER_IP_FAMILY,
+                               str,
+                               MM_BEARER_IP_FAMILY_NONE,
+                               error);
 }
 
 MMBearerAllowedAuth
@@ -975,132 +1039,60 @@ MMSmsStorage
 mm_common_get_sms_storage_from_string (const gchar *str,
                                        GError **error)
 {
-    GEnumClass *enum_class;
-    guint i;
-
-    enum_class = G_ENUM_CLASS (g_type_class_ref (MM_TYPE_SMS_STORAGE));
-
-    for (i = 0; enum_class->values[i].value_nick; i++) {
-        if (!g_ascii_strcasecmp (str, enum_class->values[i].value_nick))
-            return enum_class->values[i].value;
-    }
-
-    g_set_error (error,
-                 MM_CORE_ERROR,
-                 MM_CORE_ERROR_INVALID_ARGS,
-                 "Couldn't match '%s' with a valid MMSmsStorage value",
-                 str);
-    return MM_SMS_STORAGE_UNKNOWN;
+    return _enum_from_string (MM_TYPE_SMS_STORAGE,
+                              str,
+                              MM_SMS_STORAGE_UNKNOWN,
+                              error);
 }
 
 MMSmsCdmaTeleserviceId
 mm_common_get_sms_cdma_teleservice_id_from_string (const gchar *str,
                                                    GError **error)
 {
-    GEnumClass *enum_class;
-    guint i;
-
-    enum_class = G_ENUM_CLASS (g_type_class_ref (MM_TYPE_SMS_CDMA_TELESERVICE_ID));
-
-    for (i = 0; enum_class->values[i].value_nick; i++) {
-        if (!g_ascii_strcasecmp (str, enum_class->values[i].value_nick))
-            return enum_class->values[i].value;
-    }
-
-    g_set_error (error,
-                 MM_CORE_ERROR,
-                 MM_CORE_ERROR_INVALID_ARGS,
-                 "Couldn't match '%s' with a valid MMSmsCdmaTeleserviceId value",
-                 str);
-    return MM_SMS_CDMA_TELESERVICE_ID_UNKNOWN;
+    return _enum_from_string (MM_TYPE_SMS_CDMA_TELESERVICE_ID,
+                              str,
+                              MM_SMS_CDMA_TELESERVICE_ID_UNKNOWN,
+                              error);
 }
 
 MMSmsCdmaServiceCategory
 mm_common_get_sms_cdma_service_category_from_string (const gchar *str,
                                                      GError **error)
 {
-    GEnumClass *enum_class;
-    guint i;
-
-    enum_class = G_ENUM_CLASS (g_type_class_ref (MM_TYPE_SMS_CDMA_SERVICE_CATEGORY));
-
-    for (i = 0; enum_class->values[i].value_nick; i++) {
-        if (!g_ascii_strcasecmp (str, enum_class->values[i].value_nick))
-            return enum_class->values[i].value;
-    }
-
-    g_set_error (error,
-                 MM_CORE_ERROR,
-                 MM_CORE_ERROR_INVALID_ARGS,
-                 "Couldn't match '%s' with a valid MMSmsCdmaServiceCategory value",
-                 str);
-    return MM_SMS_CDMA_SERVICE_CATEGORY_UNKNOWN;
+    return _enum_from_string (MM_TYPE_SMS_CDMA_SERVICE_CATEGORY,
+                              str,
+                              MM_SMS_CDMA_SERVICE_CATEGORY_UNKNOWN,
+                              error);
 }
 
 MMCallDirection
 mm_common_get_call_direction_from_string (const gchar *str,
                                           GError **error)
 {
-    GEnumClass *enum_class;
-    guint i;
-
-    enum_class = G_ENUM_CLASS (g_type_class_ref (MM_TYPE_CALL_DIRECTION));
-
-    for (i = 0; enum_class->values[i].value_nick; i++) {
-        if (!g_ascii_strcasecmp (str, enum_class->values[i].value_nick))
-            return enum_class->values[i].value;
-    }
-
-    g_set_error (error,
-                 MM_CORE_ERROR,
-                 MM_CORE_ERROR_INVALID_ARGS,
-                 "Couldn't match '%s' with a valid MMCallDirection value",
-                 str);
-    return MM_CALL_DIRECTION_UNKNOWN;
+    return _enum_from_string (MM_TYPE_CALL_DIRECTION,
+                              str,
+                              MM_CALL_DIRECTION_UNKNOWN,
+                              error);
 }
 
 MMCallState
 mm_common_get_call_state_from_string (const gchar *str,
                                       GError **error)
 {
-    GEnumClass *enum_class;
-    guint i;
-
-    enum_class = G_ENUM_CLASS (g_type_class_ref (MM_TYPE_CALL_STATE));
-
-    for (i = 0; enum_class->values[i].value_nick; i++) {
-        if (!g_ascii_strcasecmp (str, enum_class->values[i].value_nick))
-            return enum_class->values[i].value;
-    }
-
-    g_set_error (error,
-                 MM_CORE_ERROR,
-                 MM_CORE_ERROR_INVALID_ARGS,
-                 "Couldn't match '%s' with a valid MMCallState value",
-                 str);
-    return MM_CALL_STATE_UNKNOWN;
+    return _enum_from_string (MM_TYPE_CALL_STATE,
+                              str,
+                              MM_CALL_STATE_UNKNOWN,
+                              error);
 }
 
 MMCallStateReason
 mm_common_get_call_state_reason_from_string (const gchar *str,
                                              GError **error)
 {
-    GEnumClass *enum_class;
-    guint i;
-
-    enum_class = G_ENUM_CLASS (g_type_class_ref (MM_TYPE_CALL_STATE_REASON));
-
-    for (i = 0; enum_class->values[i].value_nick; i++) {
-        if (!g_ascii_strcasecmp (str, enum_class->values[i].value_nick))
-            return enum_class->values[i].value;
-    }
-
-    g_set_error (error,
-                 MM_CORE_ERROR,
-                 MM_CORE_ERROR_INVALID_ARGS,
-                 "Couldn't match '%s' with a valid MMCallStateReason value",
-                 str);
-    return MM_CALL_STATE_REASON_UNKNOWN;
+    return _enum_from_string (MM_TYPE_CALL_STATE_REASON,
+                              str,
+                              MM_CALL_STATE_REASON_UNKNOWN,
+                              error);
 }
 
 MMOmaFeature
@@ -1157,22 +1149,10 @@ MMOmaSessionType
 mm_common_get_oma_session_type_from_string (const gchar *str,
                                             GError **error)
 {
-    GEnumClass *enum_class;
-    guint i;
-
-    enum_class = G_ENUM_CLASS (g_type_class_ref (MM_TYPE_OMA_SESSION_TYPE));
-
-    for (i = 0; enum_class->values[i].value_nick; i++) {
-        if (!g_ascii_strcasecmp (str, enum_class->values[i].value_nick))
-            return enum_class->values[i].value;
-    }
-
-    g_set_error (error,
-                 MM_CORE_ERROR,
-                 MM_CORE_ERROR_INVALID_ARGS,
-                 "Couldn't match '%s' with a valid MMOmaSessionType value",
-                 str);
-    return MM_OMA_SESSION_TYPE_UNKNOWN;
+    return _enum_from_string (MM_TYPE_OMA_SESSION_TYPE,
+                              str,
+                              MM_OMA_SESSION_TYPE_UNKNOWN,
+                              error);
 }
 
 GVariant *
@@ -1387,22 +1367,24 @@ mm_get_int_from_match_info (GMatchInfo *match_info,
     return ret;
 }
 
-/**
- * mm_get_uint_from_str:
- * @str: the string to convert to an unsigned int
- * @out: on success, the number
- *
- * Converts a string to an unsigned number.  All characters in the string
- * MUST be valid digits (0 - 9), otherwise FALSE is returned.
- *
- * Returns: %TRUE if the string was converted, %FALSE if it was not or if it
- * did not contain only digits.
- */
 gboolean
 mm_get_uint_from_str (const gchar *str,
-                      guint *out)
+                      guint       *out)
 {
-    gulong num;
+    guint64 num;
+
+    if (!mm_get_u64_from_str (str, &num) || num > G_MAXUINT)
+        return FALSE;
+
+    *out = (guint)num;
+    return TRUE;
+}
+
+gboolean
+mm_get_u64_from_str (const gchar *str,
+                     guint64     *out)
+{
+    guint64 num;
 
     if (!str || !str[0])
         return FALSE;
@@ -1413,33 +1395,32 @@ mm_get_uint_from_str (const gchar *str,
     }
 
     errno = 0;
-    num = strtoul (str, NULL, 10);
-    if (!errno && num <= G_MAXUINT) {
-        *out = (guint)num;
+    num = (guint64) strtoull (str, NULL, 10);
+    if (!errno) {
+        *out = num;
         return TRUE;
     }
     return FALSE;
 }
 
-/**
- * mm_get_uint_from_hex_str:
- * @str: the hex string to convert to an unsigned int
- * @out: on success, the number
- *
- * Converts a string to an unsigned number.  All characters in the string
- * MUST be valid hexadecimal digits (0-9, A-F, a-f), otherwise FALSE is
- * returned.
- *
- * An optional "0x" prefix may be given in @str.
- *
- * Returns: %TRUE if the string was converted, %FALSE if it was not or if it
- * did not contain only digits.
- */
 gboolean
 mm_get_uint_from_hex_str (const gchar *str,
                           guint       *out)
 {
-    gulong num;
+    guint64 num;
+
+    if (!mm_get_u64_from_hex_str (str, &num) || num > G_MAXUINT)
+        return FALSE;
+
+    *out = (guint)num;
+    return TRUE;
+}
+
+gboolean
+mm_get_u64_from_hex_str (const gchar *str,
+                         guint64     *out)
+{
+    guint64 num;
 
     if (!str)
         return FALSE;
@@ -1456,9 +1437,9 @@ mm_get_uint_from_hex_str (const gchar *str,
     }
 
     errno = 0;
-    num = strtoul (str, NULL, 16);
-    if (!errno && num <= G_MAXUINT) {
-        *out = (guint)num;
+    num = (guint64) strtoull (str, NULL, 16);
+    if (!errno) {
+        *out = num;
         return TRUE;
     }
     return FALSE;
@@ -1466,8 +1447,22 @@ mm_get_uint_from_hex_str (const gchar *str,
 
 gboolean
 mm_get_uint_from_match_info (GMatchInfo *match_info,
-                             guint32 match_index,
-                             guint *out)
+                             guint32     match_index,
+                             guint      *out)
+{
+    guint64 num;
+
+    if (!mm_get_u64_from_match_info (match_info, match_index, &num) || num > G_MAXUINT)
+        return FALSE;
+
+    *out = (guint)num;
+    return TRUE;
+}
+
+gboolean
+mm_get_u64_from_match_info (GMatchInfo *match_info,
+                            guint32     match_index,
+                            guint64    *out)
 {
     gchar *s;
     gboolean ret;
@@ -1476,7 +1471,7 @@ mm_get_uint_from_match_info (GMatchInfo *match_info,
     if (!s)
         return FALSE;
 
-    ret = mm_get_uint_from_str (s, out);
+    ret = mm_get_u64_from_str (s, out);
     g_free (s);
 
     return ret;
