@@ -44,17 +44,12 @@ enum {
 
 static GParamSpec *properties[PROP_LAST];
 
-typedef enum {
-    FEATURE_SUPPORT_UNKNOWN,
-    FEATURE_SUPPORTED,
-    FEATURE_UNSUPPORTED,
-} FeatureSupport;
-
 struct _MMBroadbandBearerUbloxPrivate {
     MMUbloxUsbProfile        profile;
     MMUbloxNetworkingMode    mode;
     MMUbloxBearerAllowedAuth allowed_auths;
     FeatureSupport           statistics;
+    FeatureSupport           cedata;
 };
 
 /*****************************************************************************/
@@ -327,6 +322,25 @@ dial_3gpp_finish (MMBroadbandBearer  *self,
     return MM_PORT (g_task_propagate_pointer (G_TASK (res), error));
 }
 
+static void
+cedata_activate_ready (MMBaseModem            *modem,
+                       GAsyncResult           *res,
+                       MMBroadbandBearerUblox *self)
+{
+    const gchar *response;
+    GError      *error = NULL;
+
+    response = mm_base_modem_at_command_finish (modem, res, &error);
+    if (!response) {
+        mm_warn ("ECM data connection attempt failed: %s", error->message);
+        mm_base_bearer_report_connection_status (MM_BASE_BEARER (self),
+                                                 MM_BEARER_CONNECTION_STATUS_DISCONNECTED);
+        g_error_free (error);
+    }
+    /* we received a full bearer object reference */
+    g_object_unref (self);
+}
+
 static void
 cgact_activate_ready (MMBaseModem  *modem,
                       GAsyncResult *res,
@@ -354,17 +368,82 @@ activate_3gpp (GTask *task)
 
     ctx = (CommonConnectContext *) g_task_get_task_data (task);
 
-    cmd = g_strdup_printf ("+CGACT=1,%u", ctx->cid);
-    mm_dbg ("activating PDP context #%u...", ctx->cid);
-    mm_base_modem_at_command (MM_BASE_MODEM (ctx->modem),
-                              cmd,
-                              120,
-                              FALSE,
-                              (GAsyncReadyCallback) cgact_activate_ready,
-                              task);
+    if (ctx->self->priv->profile == MM_UBLOX_USB_PROFILE_ECM &&
+        ctx->self->priv->cedata == FEATURE_SUPPORTED) {
+        /* SARA-U2xx and LISA-U20x only expose one CDC-ECM interface. Hence,
+           the fixed 0 as the interface index here. When we see modems with
+           multiple interfaces, this needs to be revisited. */
+        cmd = g_strdup_printf ("+UCEDATA=%u,0", ctx->cid);
+        mm_dbg ("establishing ECM data connection for PDP context #%u...", ctx->cid);
+        mm_base_modem_at_command (MM_BASE_MODEM (ctx->modem),
+                                  cmd,
+                                  180,
+                                  FALSE,
+                                  (GAsyncReadyCallback) cedata_activate_ready,
+                                  g_object_ref (ctx->self));
+        /* We'll mark the task done here since the modem expects the DHCP
+           discover packet while +UCEDATA runs. If the command fails, we'll
+           mark the bearer disconnected later in the callback. */
+        g_task_return_pointer (task, g_object_ref (ctx->data), g_object_unref);
+        g_object_unref (task);
+   } else {
+        cmd = g_strdup_printf ("+CGACT=1,%u", ctx->cid);
+        mm_dbg ("activating PDP context #%u...", ctx->cid);
+        mm_base_modem_at_command (MM_BASE_MODEM (ctx->modem),
+                                  cmd,
+                                  120,
+                                  FALSE,
+                                  (GAsyncReadyCallback) cgact_activate_ready,
+                                  task);
+    }
     g_free (cmd);
 }
 
+static void
+test_cedata_ready (MMBaseModem  *modem,
+                   GAsyncResult *res,
+                   GTask        *task)
+{
+    CommonConnectContext *ctx;
+    const gchar         *response;
+
+    ctx = (CommonConnectContext *) g_task_get_task_data (task);
+
+    response = mm_base_modem_at_command_finish (modem, res, NULL);
+    if (response)
+        ctx->self->priv->cedata = FEATURE_SUPPORTED;
+    else
+        ctx->self->priv->cedata = FEATURE_UNSUPPORTED;
+    mm_dbg ("u-blox: +UCEDATA command%s available",
+        (ctx->self->priv->cedata == FEATURE_SUPPORTED) ? "" : " not");
+
+    activate_3gpp (task);
+}
+
+static void
+test_cedata (GTask *task)
+{
+    CommonConnectContext *ctx;
+
+    ctx = (CommonConnectContext *) g_task_get_task_data (task);
+
+    /* We don't need to test for +UCEDATA if we're not using CDC-ECM or if we
+       have tested before. Instead, we jump right to the activation. */
+    if (ctx->self->priv->profile != MM_UBLOX_USB_PROFILE_ECM ||
+        ctx->self->priv->cedata != FEATURE_SUPPORT_UNKNOWN) {
+        activate_3gpp (task);
+        return;
+    }
+
+    mm_dbg ("u-blox: checking availability of +UCEDATA command...");
+    mm_base_modem_at_command (MM_BASE_MODEM (ctx->modem),
+                              "+UCEDATA=?",
+                              3,
+                              TRUE, /* allow_cached */
+                              (GAsyncReadyCallback) test_cedata_ready,
+                              task);
+}
+
 static void
 uauthreq_ready (MMBaseModem  *modem,
                 GAsyncResult *res,
@@ -388,7 +467,7 @@ uauthreq_ready (MMBaseModem  *modem,
         g_error_free (error);
     }
 
-    activate_3gpp (task);
+    test_cedata (task);
 }
 
 static void
@@ -457,13 +536,13 @@ out:
         cmd = g_strdup_printf ("+UAUTHREQ=%u,%u,%s,%s",
                                ctx->cid,
                                ublox_auth,
-                               quoted_password,
-                               quoted_user);
+                               quoted_user,
+                               quoted_password);
 
         g_free (quoted_user);
         g_free (quoted_password);
     } else
-        cmd = g_strdup_printf ("+UAUTHREQ=%u,0", ctx->cid);
+        cmd = g_strdup_printf ("+UAUTHREQ=%u,0,\"\",\"\"", ctx->cid);
 
     mm_dbg ("setting up authentication preferences in PDP context #%u...", ctx->cid);
     mm_base_modem_at_command (MM_BASE_MODEM (ctx->modem),
@@ -502,10 +581,10 @@ out:
             g_object_unref (task);
             return;
         }
-        /* Otherwise, ignore and jump to activate_3gpp directly as no auth setup
+        /* Otherwise, ignore and jump to test_cedata directly as no auth setup
          * is needed */
         g_error_free (error);
-        activate_3gpp (task);
+        test_cedata (task);
         return;
     }
 
@@ -669,8 +748,8 @@ ugcntrd_ready (MMBaseModem  *modem,
     MMBroadbandBearerUblox *self;
     const gchar            *response;
     GError                 *error = NULL;
-    guint                   tx_bytes = 0;
-    guint                   rx_bytes = 0;
+    guint64                 tx_bytes = 0;
+    guint64                 rx_bytes = 0;
     guint                   cid;
 
     self = MM_BROADBAND_BEARER_UBLOX (g_task_get_source_object (task));
@@ -696,7 +775,6 @@ ugcntrd_ready (MMBaseModem  *modem,
         result->bytes_tx = tx_bytes;
         g_task_return_pointer (task, result, g_free);
     }
-
     g_object_unref (task);
 }
 
@@ -879,6 +957,7 @@ mm_broadband_bearer_ublox_init (MMBroadbandBearerUblox *self)
     self->priv->mode          = MM_UBLOX_NETWORKING_MODE_UNKNOWN;
     self->priv->allowed_auths = MM_UBLOX_BEARER_ALLOWED_AUTH_UNKNOWN;
     self->priv->statistics    = FEATURE_SUPPORT_UNKNOWN;
+    self->priv->cedata        = FEATURE_SUPPORT_UNKNOWN;
 }
 
 static void
