@@ -724,9 +724,7 @@ connect_ready (MMBaseBearer *self,
         mm_dbg ("Couldn't connect bearer '%s': '%s'",
                 self->priv->path,
                 error->message);
-        if (g_error_matches (error,
-                             MM_CORE_ERROR,
-                             MM_CORE_ERROR_CANCELLED)) {
+        if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
             /* Will launch disconnection */
             launch_disconnect = TRUE;
         } else
@@ -736,10 +734,8 @@ connect_ready (MMBaseBearer *self,
     else if (g_cancellable_is_cancelled (self->priv->connect_cancellable)) {
         mm_dbg ("Connected bearer '%s', but need to disconnect", self->priv->path);
         mm_bearer_connect_result_unref (result);
-        error = g_error_new (
-            MM_CORE_ERROR,
-            MM_CORE_ERROR_CANCELLED,
-            "Bearer got connected, but had to disconnect after cancellation request");
+        error = g_error_new (G_IO_ERROR, G_IO_ERROR_CANCELLED,
+                             "Bearer got connected, but had to disconnect after cancellation request");
         launch_disconnect = TRUE;
     }
     else {
@@ -960,7 +956,7 @@ disconnect_ready (MMBaseBearer *self,
     GError *error = NULL;
 
     if (!MM_BASE_BEARER_GET_CLASS (self)->disconnect_finish (self, res, &error)) {
-        mm_dbg ("Couldn't disconnect bearer '%s'", self->priv->path);
+        mm_dbg ("Couldn't disconnect bearer '%s': %s", self->priv->path, error->message);
         bearer_update_status (self, MM_BEARER_STATUS_CONNECTED);
         g_task_return_error (task, error);
     }
@@ -1306,7 +1302,7 @@ mm_base_bearer_report_connection_status (MMBaseBearer             *self,
                                          MMBearerConnectionStatus  status)
 {
     if ((status == MM_BEARER_CONNECTION_STATUS_DISCONNECTED) && self->priv->ignore_disconnection_reports) {
-        mm_info ("ignoring disconnection report for bearer '%s'", self->priv->path);
+        mm_dbg ("ignoring disconnection report for bearer '%s'", self->priv->path);
         return;
     }
 
