@@ -20,8 +20,11 @@
 #include <string.h>
 #include <ctype.h>
 
+#define _LIBMM_INSIDE_MM
+#include <libmm-glib.h>
+
 #include "mm-charsets.h"
-#include "mm-utils.h"
+#include "mm-log.h"
 
 typedef struct {
     const char *gsm_name;
@@ -128,9 +131,8 @@ mm_modem_charset_byte_array_append (GByteArray *array,
     converted = g_convert (utf8, -1, iconv_to, "UTF-8", NULL, &written, &error);
     if (!converted) {
         if (error) {
-            g_warning ("%s: failed to convert '%s' to %s character set: (%d) %s",
-                       __func__, utf8, iconv_to,
-                       error->code, error->message);
+            mm_warn ("failed to convert '%s' to %s character set: (%d) %s",
+                     utf8, iconv_to, error->code, error->message);
             g_error_free (error);
         }
         return FALSE;
@@ -146,6 +148,31 @@ mm_modem_charset_byte_array_append (GByteArray *array,
     return TRUE;
 }
 
+gchar *
+mm_modem_charset_byte_array_to_utf8 (GByteArray     *array,
+                                     MMModemCharset  charset)
+{
+    char *converted;
+    const char *iconv_from;
+    GError *error = NULL;
+
+    g_return_val_if_fail (array != NULL, NULL);
+    g_return_val_if_fail (charset != MM_MODEM_CHARSET_UNKNOWN, NULL);
+
+    iconv_from = charset_iconv_from (charset);
+    g_return_val_if_fail (iconv_from != NULL, FALSE);
+
+    converted = g_convert ((const gchar *)array->data, array->len,
+                           "UTF-8//TRANSLIT", iconv_from,
+                           NULL, NULL, &error);
+    if (!converted || error) {
+        g_clear_error (&error);
+        converted = NULL;
+    }
+
+    return converted;
+}
+
 char *
 mm_modem_charset_hex_to_utf8 (const char *src, MMModemCharset charset)
 {
@@ -160,7 +187,7 @@ mm_modem_charset_hex_to_utf8 (const char *src, MMModemCharset charset)
     iconv_from = charset_iconv_from (charset);
     g_return_val_if_fail (iconv_from != NULL, FALSE);
 
-    unconverted = utils_hexstr2bin (src, &unconverted_len);
+    unconverted = mm_utils_hexstr2bin (src, &unconverted_len);
     if (!unconverted)
         return NULL;
 
@@ -172,10 +199,11 @@ mm_modem_charset_hex_to_utf8 (const char *src, MMModemCharset charset)
                            NULL, NULL, &error);
     if (!converted || error) {
         g_clear_error (&error);
-        g_free (unconverted);
         converted = NULL;
     }
 
+    g_free (unconverted);
+
     return converted;
 }
 
@@ -207,7 +235,7 @@ mm_modem_charset_utf8_to_hex (const char *src, MMModemCharset charset)
     }
 
     /* Get hex representation of the string */
-    hex = utils_bin2hexstr ((guint8 *)converted, converted_len);
+    hex = mm_utils_bin2hexstr ((guint8 *)converted, converted_len);
     g_free (converted);
     return hex;
 }
@@ -235,68 +263,68 @@ typedef struct GsmUtf8Mapping {
  * Mapping according to http://unicode.org/Public/MAPPINGS/ETSI/GSM0338.TXT
  */
 static const GsmUtf8Mapping gsm_def_utf8_alphabet[GSM_DEF_ALPHABET_SIZE] = {
-	/* @             £                $                ¥   */
+    /* @             £                $                ¥   */
     ONE(0x40),       TWO(0xc2, 0xa3), ONE(0x24),       TWO(0xc2, 0xa5),
     /* è             é                ù                ì   */
-	TWO(0xc3, 0xa8), TWO(0xc3, 0xa9), TWO(0xc3, 0xb9), TWO(0xc3, 0xac),
-	/* ò             Ç                \n               Ø   */
+    TWO(0xc3, 0xa8), TWO(0xc3, 0xa9), TWO(0xc3, 0xb9), TWO(0xc3, 0xac),
+    /* ò             Ç                \n               Ø   */
     TWO(0xc3, 0xb2), TWO(0xc3, 0x87), ONE(0x0a),       TWO(0xc3, 0x98),
     /* ø             \r               Å                å   */
     TWO(0xc3, 0xb8), ONE(0x0d),       TWO(0xc3, 0x85), TWO(0xc3, 0xa5),
-	/* Δ             _                Φ                Γ   */
+    /* Δ             _                Φ                Γ   */
     TWO(0xce, 0x94), ONE(0x5f),       TWO(0xce, 0xa6), TWO(0xce, 0x93),
     /* Λ             Ω                Π                Ψ   */
     TWO(0xce, 0x9b), TWO(0xce, 0xa9), TWO(0xce, 0xa0), TWO(0xce, 0xa8),
-	/* Σ             Θ                Ξ                Escape Code */
+    /* Σ             Θ                Ξ                Escape Code */
     TWO(0xce, 0xa3), TWO(0xce, 0x98), TWO(0xce, 0x9e), ONE(0xa0),
     /* Æ             æ                ß                É   */
     TWO(0xc3, 0x86), TWO(0xc3, 0xa6), TWO(0xc3, 0x9f), TWO(0xc3, 0x89),
-	/* ' '           !                "                #   */
+    /* ' '           !                "                #   */
     ONE(0x20),       ONE(0x21),       ONE(0x22),       ONE(0x23),
     /* ¤             %                &                '   */
     TWO(0xc2, 0xa4), ONE(0x25),       ONE(0x26),       ONE(0x27),
-	/* (             )                *                +   */
+    /* (             )                *                +   */
     ONE(0x28),       ONE(0x29),       ONE(0x2a),       ONE(0x2b),
     /* ,             -                .                /   */
     ONE(0x2c),       ONE(0x2d),       ONE(0x2e),       ONE(0x2f),
-	/* 0             1                2                3   */
-	ONE(0x30),       ONE(0x31),       ONE(0x32),       ONE(0x33),
+    /* 0             1                2                3   */
+    ONE(0x30),       ONE(0x31),       ONE(0x32),       ONE(0x33),
     /* 4             5                6                7   */
-	ONE(0x34),       ONE(0x35),       ONE(0x36),       ONE(0x37),
-	/* 8             9                :                ;   */
-	ONE(0x38),       ONE(0x39),       ONE(0x3a),       ONE(0x3b),
-	/* <             =                >                ?   */
-	ONE(0x3c),       ONE(0x3d),       ONE(0x3e),       ONE(0x3f),
-	/* ¡             A                B                C   */
-	TWO(0xc2, 0xa1), ONE(0x41),       ONE(0x42),       ONE(0x43),
-	/* D             E                F                G   */
-	ONE(0x44),       ONE(0x45),       ONE(0x46),       ONE(0x47),
-	/* H             I                J                K   */
-	ONE(0x48),       ONE(0x49),       ONE(0x4a),       ONE(0x4b),
-	/* L             M                N                O   */
-	ONE(0x4c),       ONE(0x4d),       ONE(0x4e),       ONE(0x4f),
-	/* P             Q                R                S   */
-	ONE(0x50),       ONE(0x51),       ONE(0x52),       ONE(0x53),
-	/* T             U                V                W   */
-	ONE(0x54),       ONE(0x55),       ONE(0x56),       ONE(0x57),
-	/* X             Y                Z                Ä   */
-	ONE(0x58),       ONE(0x59),       ONE(0x5a),       TWO(0xc3, 0x84),
-	/* Ö             Ñ                Ü                §   */
+    ONE(0x34),       ONE(0x35),       ONE(0x36),       ONE(0x37),
+    /* 8             9                :                ;   */
+    ONE(0x38),       ONE(0x39),       ONE(0x3a),       ONE(0x3b),
+    /* <             =                >                ?   */
+    ONE(0x3c),       ONE(0x3d),       ONE(0x3e),       ONE(0x3f),
+    /* ¡             A                B                C   */
+    TWO(0xc2, 0xa1), ONE(0x41),       ONE(0x42),       ONE(0x43),
+    /* D             E                F                G   */
+    ONE(0x44),       ONE(0x45),       ONE(0x46),       ONE(0x47),
+    /* H             I                J                K   */
+    ONE(0x48),       ONE(0x49),       ONE(0x4a),       ONE(0x4b),
+    /* L             M                N                O   */
+    ONE(0x4c),       ONE(0x4d),       ONE(0x4e),       ONE(0x4f),
+    /* P             Q                R                S   */
+    ONE(0x50),       ONE(0x51),       ONE(0x52),       ONE(0x53),
+    /* T             U                V                W   */
+    ONE(0x54),       ONE(0x55),       ONE(0x56),       ONE(0x57),
+    /* X             Y                Z                Ä   */
+    ONE(0x58),       ONE(0x59),       ONE(0x5a),       TWO(0xc3, 0x84),
+    /* Ö             Ñ                Ü                §   */
     TWO(0xc3, 0x96), TWO(0xc3, 0x91), TWO(0xc3, 0x9c), TWO(0xc2, 0xa7),
-	/* ¿             a                b                c   */
-	TWO(0xc2, 0xbf), ONE(0x61),       ONE(0x62),       ONE(0x63),
-	/* d             e                f                g   */
-	ONE(0x64),       ONE(0x65),       ONE(0x66),       ONE(0x67),
-	/* h             i                j                k   */
-	ONE(0x68),       ONE(0x69),       ONE(0x6a),       ONE(0x6b),
-	/* l             m                n                o   */
-	ONE(0x6c),       ONE(0x6d),       ONE(0x6e),       ONE(0x6f),
-	/* p             q                r                s   */
-	ONE(0x70),       ONE(0x71),       ONE(0x72),       ONE(0x73),
-	/* t             u                v                w   */
-	ONE(0x74),       ONE(0x75),       ONE(0x76),       ONE(0x77),
-	/* x             y                z                ä   */
-	ONE(0x78),       ONE(0x79),       ONE(0x7a),       TWO(0xc3, 0xa4),
+    /* ¿             a                b                c   */
+    TWO(0xc2, 0xbf), ONE(0x61),       ONE(0x62),       ONE(0x63),
+    /* d             e                f                g   */
+    ONE(0x64),       ONE(0x65),       ONE(0x66),       ONE(0x67),
+    /* h             i                j                k   */
+    ONE(0x68),       ONE(0x69),       ONE(0x6a),       ONE(0x6b),
+    /* l             m                n                o   */
+    ONE(0x6c),       ONE(0x6d),       ONE(0x6e),       ONE(0x6f),
+    /* p             q                r                s   */
+    ONE(0x70),       ONE(0x71),       ONE(0x72),       ONE(0x73),
+    /* t             u                v                w   */
+    ONE(0x74),       ONE(0x75),       ONE(0x76),       ONE(0x77),
+    /* x             y                z                ä   */
+    ONE(0x78),       ONE(0x79),       ONE(0x7a),       TWO(0xc3, 0xa4),
     /* ö             ñ                ü                à   */
     TWO(0xc3, 0xb6), TWO(0xc3, 0xb1), TWO(0xc3, 0xbc), TWO(0xc3, 0xa0)
 };
@@ -460,43 +488,37 @@ mm_charset_utf8_to_unpacked_gsm (const char *utf8, guint32 *out_len)
 }
 
 static gboolean
-gsm_is_subset (gunichar c, const char *utf8, gsize ulen, guint *out_clen)
+gsm_is_subset (gunichar c, const char *utf8, gsize ulen)
 {
     guint8 gsm;
 
-    *out_clen = 1;
     if (utf8_to_gsm_def_char (utf8, ulen, &gsm))
         return TRUE;
-    if (utf8_to_gsm_ext_char (utf8, ulen, &gsm)) {
-        *out_clen = 2;
+    if (utf8_to_gsm_ext_char (utf8, ulen, &gsm))
         return TRUE;
-    }
     return FALSE;
 }
 
 static gboolean
-ira_is_subset (gunichar c, const char *utf8, gsize ulen, guint *out_clen)
+ira_is_subset (gunichar c, const char *utf8, gsize ulen)
 {
-    *out_clen = 1;
     return (ulen == 1);
 }
 
 static gboolean
-ucs2_is_subset (gunichar c, const char *utf8, gsize ulen, guint *out_clen)
+ucs2_is_subset (gunichar c, const char *utf8, gsize ulen)
 {
-    *out_clen = 2;
     return (c <= 0xFFFF);
 }
 
 static gboolean
-iso88591_is_subset (gunichar c, const char *utf8, gsize ulen, guint *out_clen)
+iso88591_is_subset (gunichar c, const char *utf8, gsize ulen)
 {
-    *out_clen = 1;
     return (c <= 0xFF);
 }
 
 static gboolean
-pccp437_is_subset (gunichar c, const char *utf8, gsize ulen, guint *out_clen)
+pccp437_is_subset (gunichar c, const char *utf8, gsize ulen)
 {
     static const gunichar t[] = {
         0x00c7, 0x00fc, 0x00e9, 0x00e2, 0x00e4, 0x00e0, 0x00e5, 0x00e7, 0x00ea,
@@ -517,11 +539,9 @@ pccp437_is_subset (gunichar c, const char *utf8, gsize ulen, guint *out_clen)
     };
     int i;
 
-    *out_clen = 1;
-
     if (c <= 0x7F)
         return TRUE;
-    for (i = 0; i < sizeof (t) / sizeof (t[0]); i++) {
+    for (i = 0; i < G_N_ELEMENTS (t); i++) {
         if (c == t[i])
             return TRUE;
     }
@@ -529,7 +549,7 @@ pccp437_is_subset (gunichar c, const char *utf8, gsize ulen, guint *out_clen)
 }
 
 static gboolean
-pcdn_is_subset (gunichar c, const char *utf8, gsize ulen, guint *out_clen)
+pcdn_is_subset (gunichar c, const char *utf8, gsize ulen)
 {
     static const gunichar t[] = {
         0x00c7, 0x00fc, 0x00e9, 0x00e2, 0x00e4, 0x00e0, 0x00e5, 0x00e7, 0x00ea,
@@ -550,8 +570,6 @@ pcdn_is_subset (gunichar c, const char *utf8, gsize ulen, guint *out_clen)
     };
     int i;
 
-    *out_clen = 1;
-
     if (c <= 0x7F)
         return TRUE;
     for (i = 0; i < sizeof (t) / sizeof (t[0]); i++) {
@@ -563,7 +581,7 @@ pcdn_is_subset (gunichar c, const char *utf8, gsize ulen, guint *out_clen)
 
 typedef struct {
     MMModemCharset cs;
-    gboolean (*func) (gunichar c, const char *utf8, gsize ulen, guint *out_clen);
+    gboolean (*func) (gunichar c, const char *utf8, gsize ulen);
     guint charsize;
 } SubsetEntry;
 
@@ -578,66 +596,58 @@ SubsetEntry subset_table[] = {
 };
 
 /**
- * mm_charset_get_encoded_len:
+ * mm_charset_can_covert_to:
+ * @utf8: UTF-8 valid string.
+ * @charset: the #MMModemCharset to validate the conversion from @utf8.
  *
- * @utf8: UTF-8 valid string
- * @charset: the #MMModemCharset to check the length of @utf8 in
- * @out_unsupported: on return, number of characters of @utf8 that are not fully
- * representable in @charset
- *
- * Returns: the size in bytes of the string if converted from UTF-8 into @charset.
- **/
-guint
-mm_charset_get_encoded_len (const char *utf8,
-                            MMModemCharset charset,
-                            guint *out_unsupported)
+ * Returns: %TRUE if the conversion is possible without errors, %FALSE otherwise.
+ */
+gboolean
+mm_charset_can_convert_to (const char *utf8,
+                           MMModemCharset charset)
 {
-    const char *p = utf8, *next;
-    guint len = 0, unsupported = 0;
+    const char *p = utf8;
     SubsetEntry *e;
 
-    g_return_val_if_fail (charset != MM_MODEM_CHARSET_UNKNOWN, 0);
-    g_return_val_if_fail (utf8 != NULL, 0);
+    g_return_val_if_fail (charset != MM_MODEM_CHARSET_UNKNOWN, FALSE);
+    g_return_val_if_fail (utf8 != NULL, FALSE);
 
     if (charset == MM_MODEM_CHARSET_UTF8)
-        return strlen (utf8);
+        return TRUE;
 
     /* Find the charset in our subset table */
     for (e = &subset_table[0];
          e->cs != charset && e->cs != MM_MODEM_CHARSET_UNKNOWN;
          e++);
-    g_return_val_if_fail (e->cs != MM_MODEM_CHARSET_UNKNOWN, 0);        
+    g_return_val_if_fail (e->cs != MM_MODEM_CHARSET_UNKNOWN, FALSE);
 
     while (*p) {
         gunichar c;
         const char *end;
-        guint clen = 0;
 
         c = g_utf8_get_char_validated (p, -1);
         g_return_val_if_fail (c != (gunichar) -1, 0);
-        end = next = g_utf8_find_next_char (p, NULL);
+        end = g_utf8_find_next_char (p, NULL);
         if (end == NULL) {
-            /* Find the end... */
+            /* Find the string terminating NULL */
             end = p;
-            while (*end++);
+            while (*++end);
         }
-        
-        if (!e->func (c, p, (end - p), &clen))
-            unsupported++;
-        len += clen;
-        p = next;
+
+        if (!e->func (c, p, (end - p)))
+            return FALSE;
+
+        p = end;
     }
 
-    if (out_unsupported)
-        *out_unsupported = unsupported;
-    return len;
+    return TRUE;
 }
 
 guint8 *
-gsm_unpack (const guint8 *gsm,
-            guint32 num_septets,
-            guint8 start_offset,  /* in _bits_ */
-            guint32 *out_unpacked_len)
+mm_charset_gsm_unpack (const guint8 *gsm,
+                       guint32 num_septets,
+                       guint8 start_offset,  /* in _bits_ */
+                       guint32 *out_unpacked_len)
 {
     GByteArray *unpacked;
     int i;
@@ -670,10 +680,10 @@ gsm_unpack (const guint8 *gsm,
 }
 
 guint8 *
-gsm_pack (const guint8 *src,
-          guint32 src_len,
-          guint8 start_offset,
-          guint32 *out_packed_len)
+mm_charset_gsm_pack (const guint8 *src,
+                     guint32 src_len,
+                     guint8 start_offset,
+                     guint32 *out_packed_len)
 {
     guint8 *packed;
     guint octet = 0, lshift, plen;
@@ -818,3 +828,89 @@ mm_charset_take_and_convert_to_utf8 (gchar *str, MMModemCharset charset)
 
     return utf8;
 }
+
+/* We do all our best to convert the given string, which comes in UTF-8, to the
+ * specified charset. It may be that the output string needs to be the hex
+ * representation of the charset-encoded string, so we need to cope with that
+ * case. */
+gchar *
+mm_utf8_take_and_convert_to_charset (gchar *str,
+                                     MMModemCharset charset)
+{
+    gchar *encoded = NULL;
+
+    if (!str)
+        return NULL;
+
+    /* Validate UTF-8 always before converting */
+    if (!g_utf8_validate (str, -1, NULL)) {
+        /* Better return NULL than an invalid encoded string */
+        g_free (str);
+        return NULL;
+    }
+
+    switch (charset) {
+    case MM_MODEM_CHARSET_UNKNOWN:
+        g_warn_if_reached ();
+        encoded = str;
+        break;
+
+    case MM_MODEM_CHARSET_HEX:
+        /* FIXME: What encoding is this? */
+        g_warn_if_reached ();
+        encoded = str;
+        break;
+
+    case MM_MODEM_CHARSET_GSM:
+    case MM_MODEM_CHARSET_8859_1:
+    case MM_MODEM_CHARSET_PCCP437:
+    case MM_MODEM_CHARSET_PCDN: {
+        const gchar *iconv_to;
+        GError *error = NULL;
+
+        iconv_to = charset_iconv_from (charset);
+        encoded = g_convert (str, strlen (str),
+                             iconv_to, "UTF-8",
+                             NULL, NULL, &error);
+        if (!encoded || error) {
+            g_clear_error (&error);
+            encoded = NULL;
+        }
+
+        g_free (str);
+        break;
+    }
+
+    case MM_MODEM_CHARSET_UCS2: {
+        const gchar *iconv_to;
+        gsize encoded_len = 0;
+        GError *error = NULL;
+        gchar *hex;
+
+        iconv_to = charset_iconv_from (charset);
+        encoded = g_convert (str, strlen (str),
+                             iconv_to, "UTF-8",
+                             NULL, &encoded_len, &error);
+        if (!encoded || error) {
+            g_clear_error (&error);
+            encoded = NULL;
+        }
+
+        /* Get hex representation of the string */
+        hex = mm_utils_bin2hexstr ((guint8 *)encoded, encoded_len);
+        g_free (encoded);
+        encoded = hex;
+        g_free (str);
+        break;
+    }
+
+    /* If the given charset is ASCII or UTF8, we really expect the final string
+     * already here. */
+    case MM_MODEM_CHARSET_IRA:
+    case MM_MODEM_CHARSET_UTF8:
+        encoded = str;
+        break;
+    }
+
+    return encoded;
+}
